function [x_new,xs,scores,time] = DM_Reconstruct(...
    dictionary,y,s,beta,x0,time_limit,dictionary_pseudo)
% * Note: this code is part of the supplementary material for the paper:
% *
% * Will Landecker, Rick Chartrand and Simon DeDeo. "Robust Compressed 
% * Sensing and Sparse Coding with the Difference Map," European Conference
% * on Computer Vision (ECCV), 2014.
%
%  x = DM_Reconstruct(dictionary,y,s)
%
% [x,xs,scores,time] = DM_Reconstruct(...
%       dictionary, y, s, beta, x0, time_limit, dictionary_pseudo)
%
% This code finds a sparse reconstruction using the Difference Map
% method. 'x' is the solution, as in  y = dictionary * x.
%
%   OUTPUTS:
%
%       'xs' is a cell array of the value of 'x' at
%       each iteration.
%
%       'scores' is an array indicating the accuracy of 'x' at each 
%       iteration, meaning  || y - dictionary * x ||^2
%
%       'time' is the amount of time elapsed after every iteration.
%
%   INPUTS:
%
%       'x0' is a starting position (use [] for none). Size is N-by-1.
%
%       'beta' is the parameter for the difference-map
%       (use [] for default of -0.14).
%
%       'dictionary' is an N by M matrix: N elements, 
%       each of length M.
%
%       'y' is an M by 1 vector, to be reconstructed.
%
%       's' is the maximum L0 penalty value (number of nonzero elements)
%       in the solution.
%
%       'time_limit' is the maximum amount of time
%       that DM is allowed to run.
%
%       'dictionary_pseudo' is the pseudo-inverse of the
%       dictionary. If it is notsupplied, it will be
%       calculated. This calculating will be included in the
%       time bookkeeping.

tic;
verbose = false;

%% Check inputs

[~, n_cols] = size(dictionary);
if nargin < 3 
    error('DM_Reconstruct.m: Not enough input arguments');
end
if nargin < 4 || isempty(beta)
    beta = -0.14;
end
if nargin < 5 || isempty(x0)
    x0 = randn(n_cols,1) -0.5;
end
if nargin < 6 || isempty(time_limit)
    time_limit = inf;
end
if nargin < 7 || isempty(dictionary_pseudo)
    dictionary_pseudo = pinv(dictionary);
end




%% Options
delta_threshold  = 1e-10;       % converged if delta <= delta_threshold.




%% Initialize output variables
xs     = {};
scores = [];




%% Initialize data
x_new = x0;
delta = inf;
iter  = 0;
time  = toc;





%% Main loop
while( delta > delta_threshold )
    
    
    
    iter  = iter+1;
    x_old = x_new;
    
    
    
    %% Update x
        
    x_new = x_old + beta * ( A( fB(x_old) )  -  B( fA(x_old) ) );
    
    
    
    %% Keep track of change in x
    %
    %  We keep track of B(x_new), because we care about how well the
    %  algorithm can *sparsely* reconstruct y at each iteration.
    
    xs{iter} = B(x_new);
    delta    = sum((x_old-x_new) .^ 2);
    
    %fprintf(1,'L2 norm of update distance: %d\n', delta);
    
    
    
    
    %% How well do we reconstruct?
    %
    %  Again, we measure the reconstruction with B(x_new) because we with
    %  to measure the algorithm's ability to *sparsely* reconstruct y at
    %  each iteration.
    
    recon_dist = sum( (y - dictionary* B(x_new)).^2 );
    
    if verbose
        fprintf(1,'DM. iter %i, reconstruction distance = %0.5e\n',iter,recon_dist);
    end
    
    scores = [scores, recon_dist];
    
    
        
    
    
    
    %% Keep track of timing info
    t=toc;
    time = [time, toc];
    
    
    
    if t > time_limit
        break;
    end
    
end

if verbose
    fprintf(1,'Finished after %i iterations.\n',iter);
end




%% Output a sparse solution
x_new = B(x_new);





%% Helper functions

    function x_out = fA(x)
        Ax = A(x);
        x_out = Ax - (Ax - x)/beta;
    end



    function x_out = fB(x)
        Bx = B(x);
        x_out = Bx + (Bx - x)/beta;
    end



    function x_out = A(x_in)
        % The minimum distance projection onto the constraint set:
        %
        %   A = { x  :  y = dictionary * x }
        %
        % Derivation for this step is given in the paper.
                
        x_out = x_in - dictionary_pseudo*( dictionary*x_in - y );
             
    end



    function x_out = B(x_in)
        % The minimum distance projection onto the constraint set:
        %
        %   B = { x  :  |x|_0 <= s }
        %
        % Hard thresholding, i.e., set the n - s smallest values to 0.
        
        
        [~, inds] = sort(abs(x_in), 'descend');
        x_out = x_in;
        x_out(inds( s+1 : n_cols )) = 0;
        
        
    end



end
